using System;
using System.Collections.Generic;
using KeysightSD1;

namespace Keysight.Hvi.ProgrammingExamples.ProgExample5
{
    /// <summary>
    /// Application Configuration
    /// Defines module descriptors, configuration options and names of HVI engines, actions, triggers
    /// </summary>
    public class ApplicationConfig
    {
        #region Global Options
        /// <summary>
        /// Define PXI instruments options to open the instruments. 
        /// Complete option list can be found in the SD1 3.x User Guide.
        /// </summary>
        public static string Options { get; set; } = "channelNumbering=keysight";

        /// <summary>
        /// Simulated HW Option
        /// </summary>
        public static bool HardwareSimulated { get; set; } = true;

        /// <summary>
        /// Number of loops
        /// </summary>
        public int NumLoops { get; set; } = 5;
        #endregion

        #region Define names of HVI engines, actions, triggers
        /// <summary>
        /// AWG 
        /// </summary>
        public static string AwgEngineName { get; } = "AwgHviEngine";

        /// <summary>
        /// Digital IO Engine Name
        /// </summary>
        public static string DioEngineName { get; } = "DioHviEngine";

        /// <summary>
        /// User-defined name of the primary HVI engine
        /// </summary>
        public static string PrimaryEngineName { get; set; } = string.Format("{0}0", AwgEngineName);

        /// <summary>
        /// User-defined HVI trigger name
        /// </summary>
        public static string FpTriggerName { get; } = "FPTrigger";

        /// <summary>
        /// User-defined HVI action name: AWG trigger
        /// </summary>
        public static string AwgTriggerActionName { get; } = "AWGTrigger";

        /// <summary>
        /// User-defined HVI action name: DAQ trigger
        /// </summary>
        public static string DaqTriggerActionName { get; } = "DaqTrigger";
        #endregion

        #region Define HW Platform
        /// <summary>
        /// Define module descriptors below with your instruments information
        /// </summary>
        public static List<ModuleDescriptor> ModuleDescriptors { get; set; } = new List<ModuleDescriptor>
        {
            new ModuleDescriptor{ ApplicationRole = DioEngineName, ModelNumber = "M3202A", ChassisNumber = 2, SlotNumber = 4, Options = Options },
            new ModuleDescriptor{ ApplicationRole = AwgEngineName, ModelNumber = "M3201A", ChassisNumber = 2, SlotNumber = 10, Options = Options },
            new ModuleDescriptor{ ApplicationRole = AwgEngineName, ModelNumber = "M3202A", ChassisNumber = 2, SlotNumber = 15, Options = Options },
        };

        /// <summary>
        /// Define list of chassis containing instruments to be used by HVI
        /// </summary>
        public int[] ChassisList { get; set; } = new int[] { 2 };

        /// <summary>
        /// Define M9031A Chassis inter-connetion modules for multi-chassis setups
        /// In case of multiple chassis, chassis PXI lines need to be shared using M9031 PXI modules. 
        /// See programming example documentation for more details. For example in a three-chassis setup:
        /// M9031_descriptors = [
        /// M9031Descriptor(1, 12, 2, 12),
        /// M9031Descriptor(2, 13, 3, 12),
        /// M9031Descriptor(3, 13, 4, 12)]
        /// In case of single-chassis setup please leave the list empty: M9031_descriptors = [] 
        /// /// </summary>
        public List<M9031Descriptor> M9031Descriptors { get; set; } = new List<M9031Descriptor>
        {
            //new M9031Descriptor(1, 11, 2, 11),
        };

        /// <summary>
        /// Define PXI Trigger Resources
        /// Assign triggers to HVI object to be used for HVI-managed synchronization, data sharing, etc
        /// NOTE: In a multi-chassis setup ALL the PXI lines listed below need to be shared among each 
        /// M9031 board pair by means of SMB cable connections
        /// </summary>
        public TriggerResourceId[] PxiSyncTriggerResources { get; set; } = new TriggerResourceId[]{
            TriggerResourceId.PxiTrigger3,
            TriggerResourceId.PxiTrigger4,
            TriggerResourceId.PxiTrigger5,
            TriggerResourceId.PxiTrigger6,
            TriggerResourceId.PxiTrigger7
    };
        #endregion

        #region AWG Parameters
        // NOTE: AWG Parameters and functionalities are documented in the M32xxA AWG User Guide

        /// <summary>
        /// Path of the waveform to be queued and played by the AWG channels
        /// </summary>
        public string WaveformPath { get; set; } = "waveforms/Gaussian.csv";

        /// <summary>
        /// Sync Mode setting for all AWG channels
        /// </summary>
        public int SyncMode { get; set; } = KeysightSD1.SD_SyncModes.SYNC_NONE;

        /// <summary>
        /// Queue Mode setting for all AWG channels
        /// </summary>
        public int QueueMode { get; set; } = SD_QueueMode.ONE_SHOT;

        /// <summary>
        /// Start Delay value for the waveform queued and played by all AWG channels
        /// </summary>
        public int StartDelay { get; set; } = 0;

        /// <summary>
        /// Prescaler value applied to all the waveform queued and played by all AWG channels
        /// </summary>
        public int AwgPrescaler { get; set; } = 0;

        /// <summary>
        /// Number of cycles setting for the waveform queued and played by all AWG channels
        /// </summary>
        public int AwgNumCycles { get; set; } = 0;

        /// <summary>
        /// Amplitude setting for all AWG channels
        /// </summary>
        public int Amplitude { get; set; } = 1;

        /// <summary>
        /// Trigger mode for all AWG channels
        /// </summary>
        public int AwgTriggerMode { get; set; } = SD_TriggerModes.SWHVITRIG_CYCLE;

        /// <summary>
        /// Mask defining the AWG channels to be 
        /// </summary>
        public int ChannelMask { get; set; } = 0xF;
        #endregion
    }

}
