using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using System.Text;
using Keysight.Hvi;
using KeysightSD1;
using static Keysight.Hvi.ProgrammingExamples.ProgExample5.Program;

namespace Keysight.Hvi.ProgrammingExamples.ProgExample5
{
    /// <summary>
    /// Contains the HVI sequence programing methods for Prog. Example 5
    /// </summary>
    public static partial class ProgramExample5
    {
        /// <summary>
        /// Programs the HVI sequences of this example
        /// </summary>
        /// <param name="name">Name of the sequencer object to be defined.</param>
        /// <param name="system">SystemDefinition object defining the system where HVI runs.</param>
        /// <returns>Sequencer object to be used to program the HVI sequences.</returns>
        public static Sequencer DefineSequencer(string name, SystemDefinition system)
        {
            return new Sequencer(name, system);
        }

        /// <summary>
        /// Programs the Mixed Signal Generation Loop.
        /// </summary>
        /// <param name="sequencer">Sequencer object to be used to program the HVI sequences.</param>
        /// <param name="moduleList">List of modular instrument objects included in the HVI.</param>
        /// <param name="appConfig">User defined configuration.</param>
        public static Sequencer ProgramMixedSignalSequence(this Sequencer sequencer, List<SD1AwgModule> moduleList, ApplicationConfig appConfig)
        {
            // Check input parameters
            AssertNotNull(sequencer, nameof(sequencer));
            AssertNotNull(moduleList, nameof(moduleList));
            AssertNotNull(appConfig, nameof(appConfig));

            // Add register
            var loops = sequencer.SyncSequence.Scopes[ApplicationConfig.PrimaryEngineName].Registers.Add("Loops", RegisterSize.Short);
            loops.InitialValue = 0;
            
            // SyncWhile condition
            var syncWhileCondition = Condition.RegisterComparison(loops, ComparisonOperator.LessThan, appConfig.NumLoops);
            // Add a Sync While
            var syncWhile = sequencer.SyncSequence.AddSyncWhile("Sync Mixed-Signal Generation", 90, syncWhileCondition);
            // Define Sync While Duration
            syncWhile.Duration = new Time.Duration(1, Time.Unit.Microseconds);

            // Add a Sync Multi-Sequence Block (SMSB)
            var syncBlock = syncWhile.SyncSequence.AddSyncMultiSequenceBlock("Trigger Digital I/Os and AWGs", 260);
            // Define SMSB Duration
            syncBlock.Duration = new Time.Minimum();
            // Program the SMSB to trigger AWGs and FP pulses
            ProgramMimoTrigger(syncBlock, moduleList);

            return sequencer;
        }

        /// <summary>
        /// Programs the Sync Multi-Sequence Block to trigger AWGs and Digital Outputs
        /// </summary>
        /// <param name="syncBlock">Sync Multi-Sequence Block to be programmed.</param>
        /// <param name="moduleList">List of modular instrument objects included in the HVI.</param>
        public static void ProgramMimoTrigger(ISyncMultiSequenceBlockStatement syncBlock, List<SD1AwgModule> moduleList)
        {
            // Check input parameters
            AssertNotNull(syncBlock, nameof(syncBlock));
            AssertNotNull(moduleList, nameof(moduleList));

            // Add instructions to each local sequence within the multi-sequence block
            foreach (var sequence in syncBlock.Sequences)
            {
                // Retrieve the engine sequence from the collection of HVI Local Sequences
                // HVI Local Sequence Collection is automatically created form the user-defined HVI Engine Collection
                // Each HVI Local Sequence can be retrieved using the name alias of the corresponding HVI Engine
                string engineName = sequence.Engine.Name;

                // Program the local sequence
                if (engineName.Substring(0, engineName.Length - 1) == ApplicationConfig.DioEngineName)
                {
                    sequence.ProgramFPTrigger();
                }
                else
                {
                    sequence.ProgramAWGTrigger();
                }
                if (engineName == ApplicationConfig.PrimaryEngineName)
                {
                    sequence.ProgramIncrementCounter();
                }
            }
        }

        /// <summary>
        /// Programs an HVI local sequence to output a Front Panel (FP) pulsed signal.
        /// </summary>
        /// <param name="sequence">HVI local sequence to be programmed.</param>
        public static void ProgramFPTrigger(this ISequence sequence)
        {
            // Check input parameters
            AssertNotNull(sequence, nameof(sequence));

            // Retrieve FP Trigger prefiously defined in the HVI Trigger Collection
            var fpTrigger = sequence.Engine.Triggers[ApplicationConfig.FpTriggerName];
            
            // Retrive TriggerWrite instruction from HVI Native InstructionSet
            var triggerWrite = sequence.InstructionSet.TriggerWrite;

            // Write FP Trigger ON
            var instrTriggerOn = sequence.AddInstruction("FP Trigger ON", 10, triggerWrite.Id);
            instrTriggerOn.SetParameter(triggerWrite.Trigger.Id, fpTrigger);
            instrTriggerOn.SetParameter(triggerWrite.SyncMode.Id, triggerWrite.SyncMode.Immediate);
            instrTriggerOn.SetParameter(triggerWrite.Value.Id, triggerWrite.Value.On);

            // Write FP Trigger OFF
            var instrTriggerOff = sequence.AddInstruction("FP Trigger OFF", 100, triggerWrite.Id);
            instrTriggerOff.SetParameter(triggerWrite.Trigger.Id, fpTrigger);
            instrTriggerOff.SetParameter(triggerWrite.SyncMode.Id, triggerWrite.SyncMode.Immediate);
            instrTriggerOff.SetParameter(triggerWrite.Value.Id, triggerWrite.Value.Off);
        }

        /// <summary>
        /// Programs an HVI sequence to trigger the AWG channels
        /// </summary>
        /// <param name="sequence">HVI local sequence to be programmed.</param>
        public static void ProgramAWGTrigger(this ISequence sequence)
        {
            // Check input parameters
            AssertNotNull(sequence, nameof(sequence));

            // Execute AWG trigger from the HVI sequence of each module
            // "Action Execute" instruction executes the AWG trigger from HVI
            var instruction = sequence.AddInstruction("AWG trigger", 10, sequence.InstructionSet.ActionExecute.Id);
            instruction.SetParameter(sequence.InstructionSet.ActionExecute.Action.Id, sequence.Engine.Actions.ToArray());
        }

        /// <summary>
        /// Programs the increment of the 
        /// </summary>
        /// <param name="sequence">HVI local sequence to be programmed.</param>
        public static void ProgramIncrementCounter(this ISequence sequence)
        {
            // Check input parameters
            AssertNotNull(sequence, nameof(sequence));
            // Retrieve register object
            var loops = sequence.Scope.Registers["Loops"];

            // Increment loop counter
            var instruction = sequence.AddInstruction("Loops++", 10, sequence.InstructionSet.Add.Id);
            instruction.SetParameter(sequence.InstructionSet.Add.Destination.Id, loops);
            instruction.SetParameter(sequence.InstructionSet.Add.LeftOperand.Id, loops);
            instruction.SetParameter(sequence.InstructionSet.Add.RightOperand.Id, 1);

            // Add a delay statement to allow the register increment to complete its execution
            var instrDelay = sequence.AddDelay("Loop Delay", 50);
        }

        /// <summary>
        /// Export the programmed HVI sequences to text
        /// </summary>
        /// <param name="sequencer">Sequencer object to be used to program the HVI sequences.</param>
        public static void ExportHviSequences(Sequencer sequencer)
        {
            // Generate HVI sequence description text
            var output = sequencer.SyncSequence.ToString(OutputFormat.Debug);

            // Write text file
            string filePath = "..\\..\\..\\..\\TextOutput\\ProgExample5.txt";
            StreamWriter sw = new StreamWriter(filePath);
            sw.Write(output);
            sw.Close();
            Console.WriteLine("Programmed HVI sequences exported to file");
        }
    }
}
