using System.Collections.Generic;
using static Keysight.Hvi.ProgrammingExamples.ProgExample5.Program;

namespace Keysight.Hvi.ProgrammingExamples.ProgExample5
{
    /// <summary>
    /// Contains the system definition methods for Prog. Example 5
    /// Defines the system resources to be used by HVI
    /// </summary>
    public static partial class ProgramExample5
    {
        /// <summary>
        /// Initialize the system
        /// </summary>
        /// <param name="mySystem"></param>
        /// <returns></returns>
        public static SystemDefinition DefineSystem(string name)
        {
            return new SystemDefinition(name);
        }

        /// <summary>
        /// Define HW platform: chassis, interconnections, PXI trigger resources, synchronization, HVI clocks
        /// </summary>
        /// <param name="mySystem">SystemDefinition object defining the system where HVI runs.</param>
        /// <param name="appConfig">User defined configuration.</param>
        /// <returns></returns>
        public static SystemDefinition AddHwPlatform(this SystemDefinition mySystem, ApplicationConfig appConfig)
        {
            // Check input parameters
            AssertNotNull(mySystem, nameof(mySystem));
            AssertNotNull(appConfig, nameof(appConfig));

            //Add chassis resources
            // For multi-chassis setup details see programming example documentation
            foreach (int chassisNumber in appConfig.ChassisList)
            {
                if (ApplicationConfig.HardwareSimulated)
                {
                    mySystem.Chassis.AddWithOptions(chassisNumber, "Simulate=True,DriverSetup=model=M9018B,NoDriver=True");
                }
                else
                {
                    mySystem.Chassis.Add(chassisNumber);
                }
            }

            // Assign the defined PXI trigger resources
            mySystem.SyncResources = appConfig.PxiSyncTriggerResources;

            // Assign clock frequencies that are outside the set of the clock frequencies of each HVI engine
            // Use the code line below if you want the application to be in sync with the 10 MHz clock
            mySystem.NonHviCoreClocks = new double[] { 10e6 };

            return mySystem;
        }

        /// <summary>
        /// Define all the HVI engines to be included in the HVI
        /// </summary>
        /// <param name="mySystem">SystemDefinition object defining the system where HVI runs.</param>
        /// <param name="moduleList">List of objects representing modular instruments.</param>
        /// <returns></returns>
        public static SystemDefinition AddHviEngines(this SystemDefinition mySystem, List<SD1AwgModule> moduleList)
        {
            // Check input parameters
            AssertNotNull(mySystem, nameof(mySystem));
            AssertNotNull(moduleList, nameof(moduleList));

            // For each instrument to be used in the HVI application add its HVI Engine to the HVI Engine Collection 
            foreach (var module in moduleList)
            {
                mySystem.Engines.Add(module.Instrument.Hvi.Engines.MainEngine, module.HVIEngineName);
            }

            return mySystem;
        }

        /// <summary>
        /// Defines AWG trigger actions for each module, to be executed by the "action execute" instruction in the HVI sequence
        /// </summary>
        /// <param name="mySystem">SystemDefinition object defining the system where HVI runs.</param>
        /// <param name="moduleList">List of objects representing modular instruments.</param>
        /// <param name="appConfig">User defined configuration.</param>
        /// <returns></returns>
        public static SystemDefinition AddHviActions(this SystemDefinition mySystem, List<SD1AwgModule> moduleList, ApplicationConfig appConfig)
        {
            // Check input parameters
            AssertNotNull(mySystem, nameof(mySystem));
            AssertNotNull(moduleList, nameof(moduleList));
            AssertNotNull(appConfig, nameof(appConfig));

            // For each AWG, define the list of HVI Actions to be executed and add such list to its own HVI Action Collection
            foreach (var module in moduleList)
            {
                for (var chIndex = 1; chIndex <= module.NumChannels; chIndex++)
                {
                    // Actions need to be added to the engine's action list so that they can be executed
                    string actionName = string.Format("{0}{1}", ApplicationConfig.AwgTriggerActionName, chIndex);  // arbitrary user-defined name
                    int actionId;
                    switch (chIndex)
                    {
                        case 2:
                            actionId = module.Instrument.Hvi.Actions.Awg2Trigger;
                            break;
                        case 3:
                            actionId = module.Instrument.Hvi.Actions.Awg3Trigger;
                            break;
                        case 4:
                            actionId = module.Instrument.Hvi.Actions.Awg4Trigger;
                            break;
                        default:
                            actionId = module.Instrument.Hvi.Actions.Awg1Trigger;
                            break;
                    }
                    mySystem.Engines[module.HVIEngineName].Actions.Add(actionId, actionName);
                }
            }

            return mySystem;
        }

        /// <summary>
        /// Defines and configure the FP trigger output of each AWG
        /// </summary>
        /// <param name="mySystem">SystemDefinition object defining the system where HVI runs.</param>
        /// <param name="moduleList">List of objects representing modular instruments.</param>
        /// <param name="appConfig">User defined configuration.</param>
        /// <returns></returns>
        public static SystemDefinition AddHviTriggers(this SystemDefinition mySystem, List<SD1AwgModule> moduleList)
        {
            // Check input parameters
            AssertNotNull(mySystem, nameof(mySystem));
            AssertNotNull(moduleList, nameof(moduleList));

            // Add to the HVI Trigger Collection of each HVI Engine the FP Trigger object of that same instrument
            foreach (var module in moduleList)
            {
                int fpTriggerId = module.Instrument.Hvi.Triggers.FrontPanel1;
                ITriggerDefinition fpTrigger = mySystem.Engines[module.HVIEngineName].Triggers.Add(fpTriggerId, ApplicationConfig.FpTriggerName);
                
                // Configure FP trigger in each Hvi Engine
                fpTrigger.Config.Direction = Direction.Output;
                fpTrigger.Config.Polarity = Polarity.ActiveHigh;
                fpTrigger.Config.SyncMode = SyncMode.Immediate;
                fpTrigger.Config.HwRoutingDelay = 0;
                
                // FP TriggerMode is set to Level, which does not defines a pulse length
                // FP trigger pulse length is defined by the HVI Statements that control FP Trigger ON/OFF
                fpTrigger.Config.TriggerMode = TriggerMode.Level;
            }

            return mySystem;
        }
    }
}
